# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function
import os
import time
import argparse


## SPARK

from pyspark import SparkConf, SparkContext
from pyspark.sql.session import SparkSession

defaultPath = os.getcwd()
CLI=argparse.ArgumentParser()
CLI.add_argument(
    "--use_gpu",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--skip_sparkml",
    action = 'store_true',
    default=False
)
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
CLI.add_argument(
   "--num_solver",
   type=int,
   default= 4
)
CLI.add_argument(
   "--use_balanced_class_weights",
   action = 'store_true',
   default=False
)
args = CLI.parse_args()

if args.use_balanced_class_weights:
  class_weights = "balanced"
else:
  class_weights = None

# Set your own master (e.g 'spark://zhcc011.zurich.ibm.com:7077') or use 'local'
# Configure Spark (4 executors, local standalone mode)
# If one wants to run spark on a cluster, use your own spark configuration (set master , appName, etc...) 
#.set("spark.default.parallelism",args.num_solver)\
#.set("spark.cores.max",args.num_solver)\
conf = SparkConf()\
.set("spark.solver.count",args.num_solver)\
.setAppName("Spark-criteoKaggle2014-SparkComparison")\
.set("spark.executor.extraJavaOptions","-XX:MaxDirectMemorySize=8192m")\
.set("spark.driver.maxResultSize",8192)
# Create spark config
sc = SparkContext(conf=conf)
# Configure Spark
spark = SparkSession(sc)
n_executors=args.num_solver
# disable spark INFO logging - comment out to enable
log4j = sc._jvm.org.apache.log4j
log4j.LogManager.getRootLogger().setLevel(log4j.Level.ERROR)

n_features_ = 1000000

# data paths
# Please use the absolute path of the file if you wish to run the example on a distributed mode
filename = args.data_path + '/data/criteo.kaggle2014'
train_filename = "file://" +filename + '-train.libsvm'
test_filename = "file://" +filename + '-test.libsvm'

#from snap_ml_spark import LogisticRegression as snapml_LogisticRegression
from snap_ml_spark.estimator import LogisticRegression, LogisticRegressionModel
from snap_ml_spark.Metrics import accuracy, logisticLoss
from pyspark.ml import Pipeline
from pyspark.ml.tuning import CrossValidator, ParamGridBuilder
from pyspark.ml.evaluation import BinaryClassificationEvaluator
from pyspark.ml.evaluation import MulticlassClassificationEvaluator
		
#Sparse data frame 
train_data = spark.read.format("libsvm").option("numFeatures", str(n_features_)).load(train_filename)
test_data = spark.read.format("libsvm").option("numFeatures", str(n_features_)).load(test_filename)
# Create snapML Logistic Regression 

lr = LogisticRegression(useGpu=args.use_gpu)
pipeline = Pipeline(stages=[lr])

grid = ParamGridBuilder()\
   .addGrid(lr.regParam,[0.1, 0.01, 0.001] )\
   .build()

binEvaluator = BinaryClassificationEvaluator()

cv = CrossValidator(estimator=pipeline, estimatorParamMaps=grid, evaluator=binEvaluator)
snapml_t0 = time.time()
cvModel = cv.fit(train_data)
snapml_time = time.time() - snapml_t0
predictions=cvModel.transform(test_data)

mulEvaluator = MulticlassClassificationEvaluator(labelCol="label", predictionCol="prediction", metricName="accuracy")
snapml_accuracy = mulEvaluator.evaluate(predictions)
print('snapML  accuracy - %.4f' %snapml_accuracy, ", time: %.2f" % snapml_time, 's')

# Skip if --skip_sparkml flag is given
if args.skip_sparkml:
    print('Execution using Spark ML is skipped.')
    exit()

from pyspark.ml.classification import LogisticRegression as sparkml_LogisticRegression
	
# Create sparkML lib Logistic Regression
sparkml_lr = sparkml_LogisticRegression(standardization=False)
pipeline = Pipeline(stages=[sparkml_lr])

cv = CrossValidator(estimator=pipeline, estimatorParamMaps=grid, evaluator=binEvaluator)
sparkml_t0 = time.time()
# Fit the model and time it
cvModel = cv.fit(train_data)
sparkml_time= time.time() - sparkml_t0
# Perform inference on test data
predictions=cvModel.transform(test_data)

# Compute accuracy
sparkml_accuracy = mulEvaluator.evaluate(predictions)
# Print off Spark result
print('Spark ML accuracy',sparkml_accuracy,", time: %.2f" % sparkml_time, 's')
